import { Injectable, NotFoundException } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Equipo } from './entity/equipo.entity';
import { EquipoPrograma } from '../equipo/entity/equipo-programa.entity';
import { EquipoTipoEntrada } from '../equipo/entity/equipo-tipo-entrada.entity';
import { InstitucionProgramaService } from 'src/institucion-programa/institucion-programa.service';
import { InstitucionTipoCarritoService } from '../institucion-tipo-carrito/institucion-tipo-carrito.service';
import { InstitucionTipoEntradaService } from 'src/institucion-tipo-entrada/institucion-tipo-entrada.service';
import { ModuloService } from '../modulo/modulo.service';
import { StatusService } from 'src/status/status.service';

@Injectable()
export class EquipoService {
  constructor(
    @InjectRepository(Equipo) private equipoRepository: Repository<Equipo>,
    @InjectRepository(EquipoPrograma)
    private equipoProgramaRepository: Repository<EquipoPrograma>,
    @InjectRepository(EquipoTipoEntrada)
    private equipoTipoEntradaRepository: Repository<EquipoTipoEntrada>,
    private institucionProgramaService: InstitucionProgramaService,
    private institucionTipoCarritoService: InstitucionTipoCarritoService,
    private institucionTipoEntradaService: InstitucionTipoEntradaService,
    private moduloService: ModuloService,
    private statusService: StatusService,
  ) {}

  async create() {}

  async findAll(filtros: {
    pagina: string;
    carrito?: string;
    equipo?: string;
    id_modulo?: string;
    id_programa?: string;
    id_tipo_carrito?: string;
    id_tipo_entrada?: string;
    id_status?: string;
  }) {
    const modulo = filtros.id_modulo
      ? await this.moduloService.findById(parseInt(filtros.id_modulo))
      : null;
    const programa = filtros.id_programa
      ? await this.institucionProgramaService.findProgramaById(
          parseInt(filtros.id_programa),
        )
      : null;
    const tipoCarrito = filtros.id_tipo_carrito
      ? await this.institucionTipoCarritoService.findTipoCaritoById(
          parseInt(filtros.id_tipo_carrito),
        )
      : null;
    const tipoEntrada = filtros.id_tipo_entrada
      ? await this.institucionTipoEntradaService.findTipoEntradaById(
          parseInt(filtros.id_tipo_entrada),
        )
      : null;
    const status = filtros.id_status
      ? await this.statusService.findById(parseInt(filtros.id_status))
      : null;
    const query = this.equipoRepository
      .createQueryBuilder('e')
      .innerJoinAndSelect('e.carrito', 'c')
      .innerJoinAndSelect('e.status', 's')
      .innerJoinAndSelect('c.modulo', 'm')
      .innerJoinAndSelect('c.tipoCarrito', 'tc')
      .take(15)
      .skip((parseInt(filtros.pagina) - 1) * 15);

    if (filtros.carrito)
      query.andWhere('c.carrito LIKE :carrito', {
        carrito: `%${filtros.carrito}%`,
      });
    if (filtros.equipo)
      query.andWhere('equipo LIKE :equipo', {
        equipo: `%${filtros.equipo}%`,
      });
    if (modulo)
      query.andWhere('m.id_modulo = :id_modulo', {
        id_modulo: modulo.id_modulo,
      });
    if (programa) {
      query.innerJoinAndSelect('e.programas', 'ps');
      query.innerJoinAndSelect('ps.programa', 'p');
      query.andWhere('ps.id_programa = :id_programa', {
        id_programa: programa.id_programa,
      });
    }
    if (tipoCarrito)
      query.andWhere('tc.id_tipo_carrito = :id_tipo_carrito', {
        id_tipo_carrito: tipoCarrito.id_tipo_carrito,
      });
    if (tipoEntrada) {
      query.innerJoinAndSelect('e.tiposEntradas', 'tes');
      query.andWhere('tes.id_tipo_entrada = :id_tipo_entrada', {
        id_tipo_entrada: tipoEntrada.id_tipo_entrada,
      });
    }
    if (status)
      query.andWhere('s.id_status = :id_status', {
        id_status: status.id_status,
      });
    return query.getManyAndCount();
  }

  findById(id_equipo: number) {
    return this.equipoRepository.findOne({ id_equipo }).then((equipo) => {
      if (!equipo) throw new NotFoundException('No existe este equipo.');
      return equipo;
    });
  }

  update(attrs: Partial<Equipo>) {
    return this.findById(attrs.id_equipo)
      .then((equipo) => {
        Object.assign(equipo, attrs);
        return this.equipoRepository.save(equipo);
      })
      .then((_) => ({
        message: 'Se actualizó correctamente la información del equipo.',
      }));
  }
}
