import { Injectable, NotFoundException } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindOperator, Like, Repository } from 'typeorm';
import { Carrera } from '../institucion-carrera/entity/carrera.entity';
import { Modulo } from '../modulo/entity/modulo.entity';
import { Prestamo } from './entity/prestamo.entity';
import { TipoCarrito } from '../institucion-tipo-carrito/entity/tipo-carrito.entity';
import { TipoUsuario } from '../tipo-usuario/entity/tipo-usuario.entity';
import { CarritoService } from '../carrito/carrito.service';
import { EquipoService } from '../equipo/equipo.service';
import { InstitucionService } from '../institucion/institucion.service';
import { ModuloService } from '../modulo/modulo.service';
import { InstitucionCarreraService } from '../institucion-carrera/institucion-carrera.service';
import { InstitucionTipoCarritoService } from '../institucion-tipo-carrito/institucion-tipo-carrito.service';
import { TipoUsuarioService } from '../tipo-usuario/tipo-usuario.service';
import { UsuarioService } from '../usuario/usuario.service';

@Injectable()
export class PrestamoService {
  constructor(
    @InjectRepository(Prestamo) private repository: Repository<Prestamo>,
    private carritoService: CarritoService,
    private equipoService: EquipoService,
    private institucionService: InstitucionService,
    private institucionTipoCarritoService: InstitucionTipoCarritoService,
    private moduloService: ModuloService,
    private institucionCarreraService: InstitucionCarreraService,
    private tipoUsuarioService: TipoUsuarioService,
    private usuarioService: UsuarioService,
  ) {}

  async findAll(filtros: {
    pagina: string;
    carrito?: string;
    equipo?: string;
    id_carrera?: string;
    id_institucion?: string;
    id_modulo?: string;
    id_tipo_carrito?: string;
    id_tipo_usuario?: string;
    id_prestamo?: string;
    usuario?: string;
  }) {
    const busqueda: {
      equipo: {
        equipo?: FindOperator<string>;
        carrito: {
          carrito?: FindOperator<string>;
          modulo?: Modulo;
          tipoCarrito?: TipoCarrito;
        };
      };
      usuario: {
        usuario?: FindOperator<string>;
        tipoUsuario?: TipoUsuario;
        institucionCarrera: { carrera?: Carrera };
      };
    } = { equipo: { carrito: {} }, usuario: { institucionCarrera: {} } };
    const carrera = filtros.id_carrera
      ? await this.institucionCarreraService.findCarreraByIdCarrera(
          parseInt(filtros.id_carrera),
        )
      : null;
    const institucion = filtros.id_institucion
      ? await this.institucionService.findById(parseInt(filtros.id_institucion))
      : null;
    const modulo = filtros.id_modulo
      ? await this.moduloService.findById(parseInt(filtros.id_modulo))
      : null;
    const tipoCarrito = filtros.id_tipo_carrito
      ? await this.institucionTipoCarritoService.findTipoCaritoById(
          parseInt(filtros.id_tipo_carrito),
        )
      : null;
    const tipoUsuario = filtros.id_tipo_usuario
      ? await this.tipoUsuarioService.findById(
          parseInt(filtros.id_tipo_usuario),
        )
      : null;

    if (filtros.usuario)
      busqueda.usuario.usuario = Like(`%${filtros.usuario}%`);
    if (filtros.equipo) busqueda.equipo.equipo = Like(`%${filtros.equipo}%`);
    if (filtros.carrito)
      busqueda.equipo.carrito.carrito = Like(`%${filtros.carrito}%`);
    if (carrera) busqueda.usuario.institucionCarrera.carrera = carrera;
    if (modulo) busqueda.equipo.carrito.modulo = modulo;
    if (institucion) busqueda.equipo.carrito.modulo.institucion = institucion;
    if (tipoCarrito) busqueda.equipo.carrito.tipoCarrito = tipoCarrito;
    if (tipoUsuario) busqueda.usuario.tipoUsuario = tipoUsuario;
    return this.repository.findAndCount({
      where: busqueda,
      skip: (parseInt(filtros.pagina) - 1) * 25,
      take: 25,
    });
  }

  findById(id_prestamo: number) {
    return this.repository.findOne({ id_prestamo }).then((prestamo) => {
      if (!prestamo) throw new NotFoundException('No existe este préstamo.');
      return prestamo;
    });
  }
}
